@@ -1,9 +1,9 @@
 # coding=utf-8
 from pymycobot.common import ProtocolCode
-from pymycobot.generate import MyCobotCommandGenerator
+from pymycobot.generate import CommandGenerator
 
 
-class MyBuddyCommandGenerator(MyCobotCommandGenerator):
+class MyBuddyCommandGenerator(CommandGenerator):
     """MyBuddy Python API"""
 
     def _mesg(self, genre, *args, **kwargs):
@@ -18,7 +18,7 @@ class MyBuddyCommandGenerator(MyCobotCommandGenerator):
             **kwargs: support `has_reply`
                 has_reply: Whether there is a return value to accept.
         """
-        command_data = self._process_data_command(args)
+        command_data = self._process_data_command(genre, self.__class__.__name__, args)
 
         if genre == 178:
             # 修改wifi端口
@@ -59,14 +59,13 @@ class MyBuddyCommandGenerator(MyCobotCommandGenerator):
         """
         return self._mesg(ProtocolCode.GET_ROBOT_ID, 0, has_reply=True)
 
-    def set_robot_id(self, id, new_id):
+    def set_robot_id(self, new_id):
         """Set this robot id
         
         Args:
-            id: 0/1/2/3 (ALL/L/R/W)
             new_id: 1 - 253
         """
-        return self._mesg(ProtocolCode.SET_ROBOT_ID, id, new_id)
+        return self._mesg(ProtocolCode.SET_ROBOT_ID, new_id)
 
     # Overall status
     def power_on(self, id=0):
@@ -472,7 +471,6 @@ class MyBuddyCommandGenerator(MyCobotCommandGenerator):
         encoders_data = []
         if id == 0:
             encoders_data = (_encoders[0:7] + speed[0:7] + _encoders[7:14] + speed[7:14] + _encoders[-1:] + speed[-1:])
-            print(encoders_data)
             return self._mesg(ProtocolCode.SET_ENCODERS, _id, encoders_data)
         if id == 1 or id == 2:
             return self._mesg(ProtocolCode.SET_ENCODERS, _id, _encoders, _speed)
